@@ -691,72 +691,92 @@ globextend(const char *path, glob_t *pglob)
 }
 
 /*
- * pattern matching function for filenames.  Each occurrence of the *
- * pattern causes a recursion level.
+ * pattern matching function for filenames.
  */
 static  int
 match(const char *name, const Char *pat, const Char *patend, int m_not)
 {
     int ok, negate_range;
+    const Char *patNext;
+    const char *nameNext, *nameStart, *nameEnd;
     Char c;
 
-    while (pat < patend) {
-	size_t lwk;
+    patNext = pat;
+    nameStart = nameNext = name;
+    nameEnd = NULL;
+
+    while (pat < patend || *name) {
+	size_t lwk, pwk;
 	__Char wc, wk;
 
 	c = *pat; /* Only for M_MASK bits */
-	pat += One_Char_mbtowc(&wc, pat, MB_LEN_MAX);
+	if (*name == EOS)
+		nameEnd = name;
+
+	pwk = One_Char_mbtowc(&wc, pat, MB_LEN_MAX);
 	lwk = one_mbtowc(&wk, name, MB_LEN_MAX);
 	switch (c & M_MASK) {
 	case M_ALL:
-	    while (pat < patend && (*pat & M_MASK) == M_ALL)  /* eat consecutive '*' */
-		pat += One_Char_mbtowc(&wc, pat, MB_LEN_MAX);
-	    if (pat == patend)
-	        return (1);
-	    while (!match(name, pat, patend, m_not)) {
-		if (*name == EOS)
-		    return (0);
-		name += lwk;
-		lwk = one_mbtowc(&wk, name, MB_LEN_MAX);
+	    while ((*(pat + pwk) & M_MASK) == M_ALL) {
+		pat += pwk;
+		pwk = One_Char_mbtowc(&wc, pat, MB_LEN_MAX);
 	    }
-	    return (1);
+	    patNext = pat;
+	    nameNext = name + lwk;
+	    pat += pwk;
+	    continue;
 	case M_ONE:
 	    if (*name == EOS)
-		return (0);
+		break;
 	    name += lwk;
-	    break;
+	    pat += pwk;
+	    continue;
 	case M_SET:
 	    ok = 0;
 	    if (*name == EOS)
-		return (0);
+		break;
+	    pat += pwk;
+	    pwk = One_Char_mbtowc(&wc, pat, MB_LEN_MAX);
 	    name += lwk;
-	    if ((negate_range = ((*pat & M_MASK) == m_not)) != 0)
-		++pat;
+	    if ((negate_range = ((*pat & M_MASK) == m_not)) != 0) {
+		pat += pwk;
+		pwk = One_Char_mbtowc(&wc, pat, MB_LEN_MAX);
+	    }
 	    while ((*pat & M_MASK) != M_END) {
-		pat += One_Char_mbtowc(&wc, pat, MB_LEN_MAX);
 		if ((*pat & M_MASK) == M_RNG) {
 		    __Char wc2;
 
-		    pat++;
-		    pat += One_Char_mbtowc(&wc2, pat, MB_LEN_MAX);
+		    pat += pwk;
+		    pwk = One_Char_mbtowc(&wc2, pat, MB_LEN_MAX);
 		    if (globcharcoll(wc, wk, 0) <= 0 &&
 			globcharcoll(wk, wc2, 0) <= 0)
 			ok = 1;
 		} else if (wc == wk)
 		    ok = 1;
+		pat += pwk;
+		pwk = One_Char_mbtowc(&wc, pat, MB_LEN_MAX);
 	    }
-	    pat += One_Char_mbtowc(&wc, pat, MB_LEN_MAX);
+	    pat += pwk;
+	    pwk = One_Char_mbtowc(&wc, pat, MB_LEN_MAX);
 	    if (ok == negate_range)
-		return (0);
-	    break;
+		break;
+	    continue;
 	default:
 	    if (*name == EOS || samecase(wk) != samecase(wc))
-		return (0);
+		break;
 	    name += lwk;
-	    break;
+	    pat += pwk;
+	    continue;
 	}
+	if (nameNext != nameStart
+	    && (nameEnd == NULL || nameNext <= nameEnd)) {
+	    pat = patNext;
+	    name = nameNext;
+	    continue;
+	}
+	return 0;
     }
-    return (*name == EOS);
+    return 1;
 }
 
 /* free allocated data belonging to a glob_t structure */
