@@ -1,4 +1,3 @@
-/* $Header: /p/tcsh/cvsroot/tcsh/sh.glob.c,v 3.95 2016/08/01 16:21:09 christos Exp $ */
 /*
  * sh.glob.c: Regular expression expansion
  */
@@ -31,9 +30,6 @@
  * SUCH DAMAGE.
  */
 #include "sh.h"
-
-RCSID("$tcsh: sh.glob.c,v 3.95 2016/08/01 16:21:09 christos Exp $")
-
 #include "tc.h"
 #include "tw.h"
 
@@ -307,7 +303,7 @@ globexpand(Char **v, int noglob)
     /*
      * Step 1: expand backquotes.
      */
-    while ((s = *v++) != '\0') {
+    while ((s = *v++) != NULL) {
 	if (Strchr(s, '`')) {
 	    int     i;
 	    Char **expanded;
@@ -405,7 +401,7 @@ handleone(Char *str, Char **vl, int action)
 	for (t = vl; (p = *t++) != NULL; chars++)
 	    chars += Strlen(p);
 	str = xmalloc(chars * sizeof(Char));
-	for (t = vl, strp = str; (p = *t++) != '\0'; chars++) {
+	for (t = vl, strp = str; (p = *t++) != NULL; chars++) {
 	    while (*p)
 		 *strp++ = *p++ & TRIM;
 	    *strp++ = ' ';
@@ -583,7 +579,7 @@ rscan(Char **t, void (*f) (Char))
 {
     Char *p;
 
-    while ((p = *t++) != '\0')
+    while ((p = *t++) != NULL)
 	while (*p)
 	    (*f) (*p++);
 }
@@ -593,7 +589,7 @@ trim(Char **t)
 {
     Char *p;
 
-    while ((p = *t++) != '\0')
+    while ((p = *t++) != NULL)
 	while (*p) {
 #if INVALID_BYTE != 0
 	    if ((*p & INVALID_BYTE) != INVALID_BYTE)	/* *p < INVALID_BYTE */
@@ -610,7 +606,7 @@ tglob(Char **t)
     const Char *p;
 
     gflag = 0;
-    while ((p = *t++) != '\0') {
+    while ((p = *t++) != NULL) {
 	if (*p == '~' || *p == '=')
 	    gflag |= G_CSH;
 	else if (*p == '{' &&
