@@ -1,4 +1,3 @@
-/* $Header: /p/tcsh/cvsroot/tcsh/sh.print.c,v 3.35 2011/02/25 23:58:34 christos Exp $ */
 /*
  * sh.print.c: Primitive Output routines.
  */
@@ -31,9 +30,6 @@
  * SUCH DAMAGE.
  */
 #include "sh.h"
-
-RCSID("$tcsh: sh.print.c,v 3.35 2011/02/25 23:58:34 christos Exp $")
-
 #include "ed.h"
 
 extern int Tty_eight_bit;
@@ -231,13 +227,13 @@ flush(void)
 	return;
     if (GettingInput && !Tty_raw_mode && linp < &linbuf[sizeof linbuf - 10])
 	return;
-    if (handle_intr)
-	exitset = 1;
+    if (handle_interrupt)
+       exitset = 1;
 
     if (interrupted) {
 	interrupted = 0;
 	linp = linbuf;		/* avoid recursion as stderror calls flush */
-	if (handle_intr)
+	if (handle_interrupt)
 	    fixerror();
 	else
 	    stderror(ERR_SILENT);
@@ -284,6 +280,12 @@ flush(void)
 	 * Lost our file descriptor, exit (IRIS4D)
 	 */
 	case ESTALE:
+#endif
+#ifdef ENOENT
+	/*
+	 * Deal with SoFS bogocity: returns ENOENT instead of ESTALE.
+	 */
+	case ENOENT:
 #endif
 	/*
 	 * Over our quota, writing the history file
@@ -292,11 +294,11 @@ flush(void)
 	case EDQUOT:
 #endif
 	/* Nothing to do, but die */
-	    if (handle_intr == 0)
+	    if (handle_interrupt == 0)
 		xexit(1);
 	    /*FALLTHROUGH*/
 	default:
-	    if (handle_intr)
+	    if (handle_interrupt)
 		fixerror();
 	    else
 		stderror(ERR_SILENT);
